{% load i18n %}

<script type="text/javascript" charset="utf-8">
    DELETE_MESSAGES = ["{% trans "Really delete item?" %}", "{% trans "Confirm to delete item" %}",
        "{% trans "Item deleted successfully." %}", "{% trans "Cannot delete item" %}",
        "{% trans "Cannot delete item, because it is parent of at least one other item." %}"];
    CHANGE_TEMPLATE_MESSAGES = ["{% trans "Change template" %}",
        "{% trans "Really change template? <br />All changes are saved." %}",
        "{% trans "Really change template? <br />All changes are saved and content from <strong>%(source_regions)s</strong> is moved to <strong>%(target_region)s</strong>." %}"];

    FEINCMS_ITEM_EDITOR_GETTEXT = {
        'Hide': "{% trans "Hide" %}"
        , 'Show': "{% trans "Show" %}"
        , 'After': "{% trans "After" %}"
        , 'Before': "{% trans "Before" %}"
        , 'Insert new:': "{% trans "Insert new:" %}"
        };
</script>
