<script type="text/javascript" src="{{ STATIC_URL }}feincms/jquery-1.7.1.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}feincms/jquery-ui-1.8.16.custom.min.js"></script>
<script type="text/javascript">
    var feincms = {
        {% if FEINCMS_JQUERY_NO_CONFLICT %}
        "jQuery": jQuery.noConflict(true)
        {% else %}
        "jQuery": jQuery
        {% endif %}
    };
</script>

<div id="{{ identifier }}-new">
    {{ content|safe }}
</div>

<script type="text/javascript">
opener.feincms.fe_update_content('{{ identifier }}', feincms.jQuery('#{{ identifier }}-new').html());

window.close();
</script>
