{% extends "admin/change_form.html" %}
{% load i18n admin_modify adminmedia %}


{% block extrahead %}{{ block.super }}
<script type="text/javascript" src="{{ STATIC_URL }}feincms/jquery-1.7.1.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}feincms/jquery-ui-1.8.16.custom.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}feincms/jquery.alerts.js"></script>
<script type="text/javascript">
    var feincms = {
        {% if FEINCMS_JQUERY_NO_CONFLICT %}
        "jQuery": jQuery.noConflict(true)
        {% else %}
        "jQuery": jQuery
        {% endif %}
    };
</script>
<script type="text/javascript" src="{{ STATIC_URL }}feincms/item_editor.js"></script>

{% include "admin/feincms/_messages_js.html" %}
{% include "admin/feincms/_regions_js.html" %}

{% for inc in object.feincms_item_editor_includes.head %}{% include inc %}{% endfor %}

{% endblock %}

{% block content %}
<div class="content_main" id="frontend_editor">

<form method="post" action=".">
{% csrf_token %}
<div>
{% if is_popup %}<input type="hidden" name="_popup" value="1" />{% endif %}

    <table>
        {{ form }}
    </table>

    <div class="submit-row" >
        <input type="submit" value="{% trans "Save" %}" class="default" name="_save" />
    </div>

</div>
</form>

</div>
{% endblock %}
