{% load i18n %}
{% load feincms_tags %}
<div id="main_wrapper">
    {% for region in original.template.regions %}
        <div class="navi_tab" id="{{ region.key }}_tab">{{ region.title }}</div>
    {% endfor %}
    <div id="main">
        {% for region in original.template.regions %}
        <div id="{{ region.key }}_body" class="panel">
            <div class="empty-machine-msg">
                {% trans "Region empty" %}
            </div>
            <div class="empty-machine-msg" style="margin-left:20px; margin-top:20px;">
                {% if region.inherited %}
                    {% trans "Content from the parent site is automatically inherited. To override this behaviour, add some content." %}
                {% endif %}
            </div>

            <div class="order-machine"></div>

            <div class="machine-control">
                <div class="control-unit">
                    <span>{% trans "Add new item" %}:</span> <br/>
                    {% show_content_type_selection_widget region %}
                    <input type="button" class="order-machine-add-button button" value="Go" />
                </div>
            </div>
         </div>
        {% endfor %}
    </div>
</div>
