<div class="image-content image-content-{{ content.position }}"><img src="{{ content.get_image.url }}" alt="{{ content.alt_text }}" />{% if content.caption %}<div class="caption">{{ content.caption }}</div>{% endif %}</div>
