#changelist {
    top: 20px;
}

.navi_tab {
    float:left;
    padding: 5px 14px 5px 12px;
    cursor:pointer;
    border: 1px solid #ccc;
    border-bottom:none;
    min-width:100px;
    margin-top:3px;
    /* height:13px; */
    font-weight: bold;
    font-size: 12px;
    background-image:none;
    color: #333333;
    background: #E1E1E1 url(img/nav-bg.gif) top repeat-x;
}
.tab_active {
    margin-top: 0;
    padding-top: 6px;
    padding-bottom: 7px;
    background: #7BA0C6 url('img/default-bg.gif') top repeat-x;
    color: white;
}

#main {
    clear:both;
    padding: 10px 10px 10px 10px;
    border: 1px solid #ccc;
    margin: 0 0 10px 0;
}
.panel {
     display:none;
     position:relative;
     padding-bottom:50px;
}

.order-machine {
    margin-bottom:10px;
}

.order-item {
    margin: 0 0 10px 0;
    position:relative;
}

.order-item .handle {
    position: absolute;
    top: 0;
    left: 0;
    display: block;
    height: 24px;
    width: 15px;
    cursor: move;
}

.order-item .collapse {
    cursor: pointer;
    color: #444;
    font-weight: normal;
}

.item-delete {
    cursor:pointer;
    float:right;
    margin: 2px 0px 0px 0;
}

.highlight, .helper {
    height: 25px;
    margin: 0 0 10px 0;
    border: none;
    opacity: 0.3;
    background: url('img/item_editor_form_title.png') no-repeat left bottom;
}

.helper{
    height: 25px !important;
    opacity: 1;
}

.button {
    margin:5px; padding:5px;
    font-weight: bold;
    background-image:url('img/nav-bg.gif');
    cursor:pointer;
    border: 1px solid #678;
}

th { vertical-align: middle; }

select {
    max-width: 580px;
}

#main_wrapper {
    margin: 10px 0 30px 0;
}

textarea {
    width: 580px;
    margin-top:5px;
    margin-bottom:5px;
}

.inline-group .tabular textarea {
    width: auto;
}

.item-minimize {
    width: 15px;
    height:15px;
    float: left;
    cursor: pointer;
    margin-left:-17px;
}

.item-minimize-disabled {
    width: 15px;
    height:15px;
    float: left;
    margin-left:-17px;
}

.item-controls {
    background:  url('img/wrench.png') no-repeat 12px center;
    padding-left:27px;
    height: 35px;
}

.item-control-unit {
    float:left;
    padding: 3px 10px 0 7px;
    border-left: 1px solid #eee;
}

.item-control-unit:first-child {
    border-left: none;
}

.machine-control {
    height:70px;
    padding: 5px 10px 5px 10px;
    border: 1px solid #ccc;
    background-color: #edf3fe;
    position:absolute;
    left:-11px;
    bottom:-30px;
    width: 100%;
}

.control-unit {
    float:left;
    padding-left: 5px;
    padding-right:20px;
    border-left: 1px solid #eee;
}

.control-unit:first-child {
    border-left: none;
}

.control-unit span {
    font-weight:bold;
}

a.actionbutton {
    display: block;
    background-repeat: no-repeat;
    width:50px;
    height:50px;
    float:left;
    margin: 5px 20px 0 0;
    text-indent:-7000px;
}

a.richtextcontent { background: url(img/contenttypes.png) no-repeat 0 0; }
a.richtextcontent:hover { background-position: 0 -70px; }

a.imagecontent { background: url(img/contenttypes.png) no-repeat -70px 0; }
a.imagecontent:hover { background-position: -70px -70px; }

a.gallerycontent { background: url(img/contenttypes.png) no-repeat -140px 0; }
a.gallerycontent:hover { background-position: -140px -70px; }

a.oembedcontent { background: url(img/contenttypes.png) no-repeat -280px 0; }
a.oembedcontent:hover { background-position: -280px -70px; }

a.pdfcontent { background: url(img/contenttypes.png) no-repeat -210px 0; }
a.pdfcontent:hover { background-position: -210px -70px; }

a.audiocontent { background: url(img/contenttypes.png) no-repeat -350px 0; }
a.audiocontent:hover { background-position: -350px -70px; }

.control-unit select,
.control-unit input {
    position: relative;
    top: 15px;
}

.empty-machine-msg {
    margin:10px 0px 5px 300px;
    font-style: italic;
    font-size:14px;
    color:#999;
}

td span select {
    width:600px;
}

#popup_bg {
    width:100%; height:100%;
    background-color:white;
    opacity: 0.5;
    filter:alpha(opacity=50);
    position:absolute;
    top:0px; left:0px;
}


/* jQuery alerts */
#popup_container {
    font-family: Arial, sans-serif;
    font-size: 12px;
    min-width: 300px; /* Dialog will be no smaller than this */
    max-width: 600px; /* Dialog will wrap after this width */
    background: #FFF;
    border: solid 1px #666;
    color: #000;
}

#popup_title {
    font-size: 14px;
    font-weight: bold;
    text-align: center;
    line-height: 1.75em;
    color: #666;
    background: #eee url(img/title.gif) top repeat-x;
    border: solid 1px #FFF;
    border-bottom: solid 1px #666;
    cursor: default;
    padding: 0em;
    margin: 0em;
}

#popup_content {
    background: 16px 16px no-repeat url(img/info.gif);
    padding: 1em 1.75em;
    margin: 0em;
}

#popup_content.alert {
    background-image: url(img/info.gif);
}

#popup_content.confirm {
    background-image: url(img/important.gif);
}

#popup_content.prompt {
    background-image: url(img/help.gif);
}

#popup_message {
    padding-left: 48px;
}

#popup_panel {
    text-align: center;
    margin: 1em 0em 0em 1em;
}

#popup_prompt {
    margin: .5em 0em;
}


/* override Django admin styles */
.order-machine fieldset.module > h2{
    padding: 4px 5px 6px 17px;
    background: url(img/item_editor_form_title.png) no-repeat left bottom;
}

.change-template-button {
    margin-left: 7em;
    padding-left: 30px;
}

/* Allow nested lists in error items */
ul.errorlist ul {
	margin-left: 1em;
	padding-left: 0;
	list-style-type: square;
}

ul.errorlist li li {
	/* Avoid repeating the warning image every time*/
	background-image:none;
	padding: 0;
}

/* tree editor */

.page_marker { display:block; text-align: right; float: left; padding: 1px 1px 0 0; }
tr.cut { border: 2px dashed black; opacity: 0.5; }
.title-column { width: 400px; }

table#result_list :focus { background-color: #ccddff !important; outline: 0px; }

#page_form {
    clear: both;
}

div.order-machine div.inline-related > h3{
    display: none;
}

.hidden-form-row{
    display: none;
}
