<ul class="breadcrumbs">
    {% for url, title in trail %}
        <li>
            {% if url %}
                <a href="{{ url }}">{{ title }}</a>
            {% else %}
                {{ title }}
            {% endif %}
        </li>
    {% endfor %}
</ul>