{% load i18n %}
{% load url from future %}

<script type="text/javascript" charset="utf-8">
(function () {
    // load jQuery if not yet present
    if (typeof(feincms) == "undefined") {
        var jquery_url = "{{ STATIC_URL }}feincms/jquery-1.7.1.min.js";
        document.write(unescape('%3Cscript src="' + jquery_url + '" type="text/javascript"%3E%3C/script%3E'));
    }
})();
</script>
<script type="text/javascript">
    var feincms = {
        "admin_index": "{% url "admin:index" %}",
        {% if FEINCMS_JQUERY_NO_CONFLICT %}
        "jQuery": jQuery.noConflict(true)
        {% else %}
        "jQuery": jQuery
        {% endif %}
    };
</script>

<script src="{{ STATIC_URL }}feincms//frontend_editing.js" type="text/javascript" charset="utf-8"></script>
<link rel="stylesheet" href="{{ STATIC_URL }}feincms//frontend_editing.css" type="text/css" media="screen" charset="utf-8">

<div id="fe_controls">
    <a href="?frontend_editing=0">{% trans "Stop Editing" %}</a>
    <a href="{% url "admin:page_page_change" feincms_page.pk %}">Admin Page</a>
</div>

<div id="fe_tools">
    <a id="fe_tools_edit" href="#">{% trans "edit" %}</a>
    <!--
    <a id="fe_tools_new" href="#">{% trans "new" %}</a>
    <a id="fe_tools_up" href="#">{% trans "up" %}</a>
    <a id="fe_tools_down" href="#">{% trans "down" %}</a>
    <a id="fe_tools_remove" href="#">{% trans "remove" %}</a>
    -->
</div>
